package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class ContainerStateBuilder extends ContainerStateFluentImpl<ContainerStateBuilder> implements VisitableBuilder<ContainerState,ContainerStateBuilder>{

    ContainerStateFluent<?> fluent;
    Boolean validationEnabled;

    public ContainerStateBuilder(){
            this(true);
    }
    public ContainerStateBuilder(Boolean validationEnabled){
            this(new ContainerState(), validationEnabled);
    }
    public ContainerStateBuilder(ContainerStateFluent<?> fluent){
            this(fluent, true);
    }
    public ContainerStateBuilder(ContainerStateFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new ContainerState(), validationEnabled);
    }
    public ContainerStateBuilder(ContainerStateFluent<?> fluent,ContainerState instance){
            this(fluent, instance, true);
    }
    public ContainerStateBuilder(ContainerStateFluent<?> fluent,ContainerState instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withRunning(instance.getRunning()); 
            fluent.withTerminated(instance.getTerminated()); 
            fluent.withWaiting(instance.getWaiting()); 
            this.validationEnabled = validationEnabled; 
    }
    public ContainerStateBuilder(ContainerState instance){
            this(instance,true);
    }
    public ContainerStateBuilder(ContainerState instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withRunning(instance.getRunning()); 
            this.withTerminated(instance.getTerminated()); 
            this.withWaiting(instance.getWaiting()); 
            this.validationEnabled = validationEnabled; 
    }

    public ContainerState build(){
            ContainerState buildable = new ContainerState(fluent.getRunning(),fluent.getTerminated(),fluent.getWaiting());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            ContainerStateBuilder that = (ContainerStateBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
