package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class GlusterfsVolumeSourceBuilder extends GlusterfsVolumeSourceFluentImpl<GlusterfsVolumeSourceBuilder> implements VisitableBuilder<GlusterfsVolumeSource,GlusterfsVolumeSourceBuilder>{

    GlusterfsVolumeSourceFluent<?> fluent;
    Boolean validationEnabled;

    public GlusterfsVolumeSourceBuilder(){
            this(true);
    }
    public GlusterfsVolumeSourceBuilder(Boolean validationEnabled){
            this(new GlusterfsVolumeSource(), validationEnabled);
    }
    public GlusterfsVolumeSourceBuilder(GlusterfsVolumeSourceFluent<?> fluent){
            this(fluent, true);
    }
    public GlusterfsVolumeSourceBuilder(GlusterfsVolumeSourceFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new GlusterfsVolumeSource(), validationEnabled);
    }
    public GlusterfsVolumeSourceBuilder(GlusterfsVolumeSourceFluent<?> fluent,GlusterfsVolumeSource instance){
            this(fluent, instance, true);
    }
    public GlusterfsVolumeSourceBuilder(GlusterfsVolumeSourceFluent<?> fluent,GlusterfsVolumeSource instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withEndpoints(instance.getEndpoints()); 
            fluent.withPath(instance.getPath()); 
            fluent.withReadOnly(instance.getReadOnly()); 
            this.validationEnabled = validationEnabled; 
    }
    public GlusterfsVolumeSourceBuilder(GlusterfsVolumeSource instance){
            this(instance,true);
    }
    public GlusterfsVolumeSourceBuilder(GlusterfsVolumeSource instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withEndpoints(instance.getEndpoints()); 
            this.withPath(instance.getPath()); 
            this.withReadOnly(instance.getReadOnly()); 
            this.validationEnabled = validationEnabled; 
    }

    public GlusterfsVolumeSource build(){
            GlusterfsVolumeSource buildable = new GlusterfsVolumeSource(fluent.getEndpoints(),fluent.getPath(),fluent.isReadOnly());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            GlusterfsVolumeSourceBuilder that = (GlusterfsVolumeSourceBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
