package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class HTTPHeaderBuilder extends HTTPHeaderFluentImpl<HTTPHeaderBuilder> implements VisitableBuilder<HTTPHeader,HTTPHeaderBuilder>{

    HTTPHeaderFluent<?> fluent;
    Boolean validationEnabled;

    public HTTPHeaderBuilder(){
            this(true);
    }
    public HTTPHeaderBuilder(Boolean validationEnabled){
            this(new HTTPHeader(), validationEnabled);
    }
    public HTTPHeaderBuilder(HTTPHeaderFluent<?> fluent){
            this(fluent, true);
    }
    public HTTPHeaderBuilder(HTTPHeaderFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new HTTPHeader(), validationEnabled);
    }
    public HTTPHeaderBuilder(HTTPHeaderFluent<?> fluent,HTTPHeader instance){
            this(fluent, instance, true);
    }
    public HTTPHeaderBuilder(HTTPHeaderFluent<?> fluent,HTTPHeader instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withName(instance.getName()); 
            fluent.withValue(instance.getValue()); 
            this.validationEnabled = validationEnabled; 
    }
    public HTTPHeaderBuilder(HTTPHeader instance){
            this(instance,true);
    }
    public HTTPHeaderBuilder(HTTPHeader instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withName(instance.getName()); 
            this.withValue(instance.getValue()); 
            this.validationEnabled = validationEnabled; 
    }

    public HTTPHeader build(){
            HTTPHeader buildable = new HTTPHeader(fluent.getName(),fluent.getValue());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            HTTPHeaderBuilder that = (HTTPHeaderBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
