package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class IDRangeBuilder extends IDRangeFluentImpl<IDRangeBuilder> implements VisitableBuilder<IDRange,IDRangeBuilder>{

    IDRangeFluent<?> fluent;
    Boolean validationEnabled;

    public IDRangeBuilder(){
            this(true);
    }
    public IDRangeBuilder(Boolean validationEnabled){
            this(new IDRange(), validationEnabled);
    }
    public IDRangeBuilder(IDRangeFluent<?> fluent){
            this(fluent, true);
    }
    public IDRangeBuilder(IDRangeFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new IDRange(), validationEnabled);
    }
    public IDRangeBuilder(IDRangeFluent<?> fluent,IDRange instance){
            this(fluent, instance, true);
    }
    public IDRangeBuilder(IDRangeFluent<?> fluent,IDRange instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withMax(instance.getMax()); 
            fluent.withMin(instance.getMin()); 
            this.validationEnabled = validationEnabled; 
    }
    public IDRangeBuilder(IDRange instance){
            this(instance,true);
    }
    public IDRangeBuilder(IDRange instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withMax(instance.getMax()); 
            this.withMin(instance.getMin()); 
            this.validationEnabled = validationEnabled; 
    }

    public IDRange build(){
            IDRange buildable = new IDRange(fluent.getMax(),fluent.getMin());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            IDRangeBuilder that = (IDRangeBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
