package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class ServiceSpecBuilder extends ServiceSpecFluentImpl<ServiceSpecBuilder> implements VisitableBuilder<ServiceSpec,ServiceSpecBuilder>{

    ServiceSpecFluent<?> fluent;
    Boolean validationEnabled;

    public ServiceSpecBuilder(){
            this(true);
    }
    public ServiceSpecBuilder(Boolean validationEnabled){
            this(new ServiceSpec(), validationEnabled);
    }
    public ServiceSpecBuilder(ServiceSpecFluent<?> fluent){
            this(fluent, true);
    }
    public ServiceSpecBuilder(ServiceSpecFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new ServiceSpec(), validationEnabled);
    }
    public ServiceSpecBuilder(ServiceSpecFluent<?> fluent,ServiceSpec instance){
            this(fluent, instance, true);
    }
    public ServiceSpecBuilder(ServiceSpecFluent<?> fluent,ServiceSpec instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withClusterIP(instance.getClusterIP()); 
            fluent.withDeprecatedPublicIPs(instance.getDeprecatedPublicIPs()); 
            fluent.withExternalIPs(instance.getExternalIPs()); 
            fluent.withLoadBalancerIP(instance.getLoadBalancerIP()); 
            fluent.withLoadBalancerSourceRanges(instance.getLoadBalancerSourceRanges()); 
            fluent.withPortalIP(instance.getPortalIP()); 
            fluent.withPorts(instance.getPorts()); 
            fluent.withSelector(instance.getSelector()); 
            fluent.withSessionAffinity(instance.getSessionAffinity()); 
            fluent.withType(instance.getType()); 
            this.validationEnabled = validationEnabled; 
    }
    public ServiceSpecBuilder(ServiceSpec instance){
            this(instance,true);
    }
    public ServiceSpecBuilder(ServiceSpec instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withClusterIP(instance.getClusterIP()); 
            this.withDeprecatedPublicIPs(instance.getDeprecatedPublicIPs()); 
            this.withExternalIPs(instance.getExternalIPs()); 
            this.withLoadBalancerIP(instance.getLoadBalancerIP()); 
            this.withLoadBalancerSourceRanges(instance.getLoadBalancerSourceRanges()); 
            this.withPortalIP(instance.getPortalIP()); 
            this.withPorts(instance.getPorts()); 
            this.withSelector(instance.getSelector()); 
            this.withSessionAffinity(instance.getSessionAffinity()); 
            this.withType(instance.getType()); 
            this.validationEnabled = validationEnabled; 
    }

    public ServiceSpec build(){
            ServiceSpec buildable = new ServiceSpec(fluent.getClusterIP(),fluent.getDeprecatedPublicIPs(),fluent.getExternalIPs(),fluent.getLoadBalancerIP(),fluent.getLoadBalancerSourceRanges(),fluent.getPortalIP(),fluent.getPorts(),fluent.getSelector(),fluent.getSessionAffinity(),fluent.getType());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            ServiceSpecBuilder that = (ServiceSpecBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
