package io.fabric8.kubernetes.api.model.extensions;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class ScaleStatusBuilder extends ScaleStatusFluentImpl<ScaleStatusBuilder> implements VisitableBuilder<ScaleStatus,ScaleStatusBuilder>{

    ScaleStatusFluent<?> fluent;
    Boolean validationEnabled;

    public ScaleStatusBuilder(){
            this(true);
    }
    public ScaleStatusBuilder(Boolean validationEnabled){
            this(new ScaleStatus(), validationEnabled);
    }
    public ScaleStatusBuilder(ScaleStatusFluent<?> fluent){
            this(fluent, true);
    }
    public ScaleStatusBuilder(ScaleStatusFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new ScaleStatus(), validationEnabled);
    }
    public ScaleStatusBuilder(ScaleStatusFluent<?> fluent,ScaleStatus instance){
            this(fluent, instance, true);
    }
    public ScaleStatusBuilder(ScaleStatusFluent<?> fluent,ScaleStatus instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withReplicas(instance.getReplicas()); 
            fluent.withSelector(instance.getSelector()); 
            fluent.withTargetSelector(instance.getTargetSelector()); 
            this.validationEnabled = validationEnabled; 
    }
    public ScaleStatusBuilder(ScaleStatus instance){
            this(instance,true);
    }
    public ScaleStatusBuilder(ScaleStatus instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withReplicas(instance.getReplicas()); 
            this.withSelector(instance.getSelector()); 
            this.withTargetSelector(instance.getTargetSelector()); 
            this.validationEnabled = validationEnabled; 
    }

    public ScaleStatus build(){
            ScaleStatus buildable = new ScaleStatus(fluent.getReplicas(),fluent.getSelector(),fluent.getTargetSelector());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            ScaleStatusBuilder that = (ScaleStatusBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
