package io.fabric8.kubernetes.api.model.extensions;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class SubresourceReferenceBuilder extends SubresourceReferenceFluentImpl<SubresourceReferenceBuilder> implements VisitableBuilder<SubresourceReference,SubresourceReferenceBuilder>{

    SubresourceReferenceFluent<?> fluent;
    Boolean validationEnabled;

    public SubresourceReferenceBuilder(){
            this(true);
    }
    public SubresourceReferenceBuilder(Boolean validationEnabled){
            this(new SubresourceReference(), validationEnabled);
    }
    public SubresourceReferenceBuilder(SubresourceReferenceFluent<?> fluent){
            this(fluent, true);
    }
    public SubresourceReferenceBuilder(SubresourceReferenceFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new SubresourceReference(), validationEnabled);
    }
    public SubresourceReferenceBuilder(SubresourceReferenceFluent<?> fluent,SubresourceReference instance){
            this(fluent, instance, true);
    }
    public SubresourceReferenceBuilder(SubresourceReferenceFluent<?> fluent,SubresourceReference instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withApiVersion(instance.getApiVersion()); 
            fluent.withKind(instance.getKind()); 
            fluent.withName(instance.getName()); 
            fluent.withSubresource(instance.getSubresource()); 
            this.validationEnabled = validationEnabled; 
    }
    public SubresourceReferenceBuilder(SubresourceReference instance){
            this(instance,true);
    }
    public SubresourceReferenceBuilder(SubresourceReference instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withApiVersion(instance.getApiVersion()); 
            this.withKind(instance.getKind()); 
            this.withName(instance.getName()); 
            this.withSubresource(instance.getSubresource()); 
            this.validationEnabled = validationEnabled; 
    }

    public SubresourceReference build(){
            SubresourceReference buildable = new SubresourceReference(fluent.getApiVersion(),fluent.getKind(),fluent.getName(),fluent.getSubresource());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            SubresourceReferenceBuilder that = (SubresourceReferenceBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
