package io.fabric8.openshift.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class RouteSpecBuilder extends RouteSpecFluentImpl<RouteSpecBuilder> implements VisitableBuilder<RouteSpec,RouteSpecBuilder>{

    RouteSpecFluent<?> fluent;
    Boolean validationEnabled;

    public RouteSpecBuilder(){
            this(true);
    }
    public RouteSpecBuilder(Boolean validationEnabled){
            this(new RouteSpec(), validationEnabled);
    }
    public RouteSpecBuilder(RouteSpecFluent<?> fluent){
            this(fluent, true);
    }
    public RouteSpecBuilder(RouteSpecFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new RouteSpec(), validationEnabled);
    }
    public RouteSpecBuilder(RouteSpecFluent<?> fluent,RouteSpec instance){
            this(fluent, instance, true);
    }
    public RouteSpecBuilder(RouteSpecFluent<?> fluent,RouteSpec instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withAlternateBackends(instance.getAlternateBackends()); 
            fluent.withHost(instance.getHost()); 
            fluent.withPath(instance.getPath()); 
            fluent.withPort(instance.getPort()); 
            fluent.withTls(instance.getTls()); 
            fluent.withTo(instance.getTo()); 
            this.validationEnabled = validationEnabled; 
    }
    public RouteSpecBuilder(RouteSpec instance){
            this(instance,true);
    }
    public RouteSpecBuilder(RouteSpec instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withAlternateBackends(instance.getAlternateBackends()); 
            this.withHost(instance.getHost()); 
            this.withPath(instance.getPath()); 
            this.withPort(instance.getPort()); 
            this.withTls(instance.getTls()); 
            this.withTo(instance.getTo()); 
            this.validationEnabled = validationEnabled; 
    }

    public RouteSpec build(){
            RouteSpec buildable = new RouteSpec(fluent.getAlternateBackends(),fluent.getHost(),fluent.getPath(),fluent.getPort(),fluent.getTls(),fluent.getTo());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            RouteSpecBuilder that = (RouteSpecBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
