package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.BaseFluent;
import java.lang.String;
import javax.validation.constraints.NotNull;
import com.fasterxml.jackson.annotation.JsonProperty;
import java.lang.Integer;
import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import javax.validation.Valid;
import io.fabric8.kubernetes.api.model.validators.CheckObjectMeta;
import java.lang.Object;
import io.fabric8.kubernetes.api.builder.Nested;

public class EventFluentImpl<A extends EventFluent<A>> extends BaseFluent<A> implements EventFluent<A>{

    private String apiVersion;
    private Integer count;
    private String firstTimestamp;
    private VisitableBuilder<? extends ObjectReference,?> involvedObject;
    private String kind;
    private String lastTimestamp;
    private String message;
    private VisitableBuilder<? extends ObjectMeta,?> metadata;
    private String reason;
    private VisitableBuilder<? extends EventSource,?> source;
    private String type;

    public EventFluentImpl(){
    }
    public EventFluentImpl(Event instance){
            this.withApiVersion(instance.getApiVersion()); 
            this.withCount(instance.getCount()); 
            this.withFirstTimestamp(instance.getFirstTimestamp()); 
            this.withInvolvedObject(instance.getInvolvedObject()); 
            this.withKind(instance.getKind()); 
            this.withLastTimestamp(instance.getLastTimestamp()); 
            this.withMessage(instance.getMessage()); 
            this.withMetadata(instance.getMetadata()); 
            this.withReason(instance.getReason()); 
            this.withSource(instance.getSource()); 
            this.withType(instance.getType()); 
    }

    public String getApiVersion(){
            return this.apiVersion;
    }

    public A withApiVersion(String apiVersion){
            this.apiVersion=apiVersion; return (A) this;
    }

    public Integer getCount(){
            return this.count;
    }

    public A withCount(Integer count){
            this.count=count; return (A) this;
    }

    public String getFirstTimestamp(){
            return this.firstTimestamp;
    }

    public A withFirstTimestamp(String firstTimestamp){
            this.firstTimestamp=firstTimestamp; return (A) this;
    }

    public ObjectReference getInvolvedObject(){
            return this.involvedObject!=null?this.involvedObject.build():null;
    }

    public A withInvolvedObject(ObjectReference involvedObject){
            if (involvedObject!=null){ this.involvedObject= new ObjectReferenceBuilder(involvedObject); _visitables.add(this.involvedObject);} return (A) this;
    }

    public EventFluent.InvolvedObjectNested<A> withNewInvolvedObject(){
            return new InvolvedObjectNestedImpl();
    }

    public EventFluent.InvolvedObjectNested<A> withNewInvolvedObjectLike(ObjectReference item){
            return new InvolvedObjectNestedImpl(item);
    }

    public EventFluent.InvolvedObjectNested<A> editInvolvedObject(){
            return withNewInvolvedObjectLike(getInvolvedObject());
    }

    public String getKind(){
            return this.kind;
    }

    public A withKind(String kind){
            this.kind=kind; return (A) this;
    }

    public String getLastTimestamp(){
            return this.lastTimestamp;
    }

    public A withLastTimestamp(String lastTimestamp){
            this.lastTimestamp=lastTimestamp; return (A) this;
    }

    public String getMessage(){
            return this.message;
    }

    public A withMessage(String message){
            this.message=message; return (A) this;
    }

    public ObjectMeta getMetadata(){
            return this.metadata!=null?this.metadata.build():null;
    }

    public A withMetadata(ObjectMeta metadata){
            if (metadata!=null){ this.metadata= new ObjectMetaBuilder(metadata); _visitables.add(this.metadata);} return (A) this;
    }

    public EventFluent.MetadataNested<A> withNewMetadata(){
            return new MetadataNestedImpl();
    }

    public EventFluent.MetadataNested<A> withNewMetadataLike(ObjectMeta item){
            return new MetadataNestedImpl(item);
    }

    public EventFluent.MetadataNested<A> editMetadata(){
            return withNewMetadataLike(getMetadata());
    }

    public String getReason(){
            return this.reason;
    }

    public A withReason(String reason){
            this.reason=reason; return (A) this;
    }

    public EventSource getSource(){
            return this.source!=null?this.source.build():null;
    }

    public A withSource(EventSource source){
            if (source!=null){ this.source= new EventSourceBuilder(source); _visitables.add(this.source);} return (A) this;
    }

    public EventFluent.SourceNested<A> withNewSource(){
            return new SourceNestedImpl();
    }

    public EventFluent.SourceNested<A> withNewSourceLike(EventSource item){
            return new SourceNestedImpl(item);
    }

    public EventFluent.SourceNested<A> editSource(){
            return withNewSourceLike(getSource());
    }

    public A withNewSource(String component,String host){
            return (A)withSource(new EventSource(component, host));
    }

    public String getType(){
            return this.type;
    }

    public A withType(String type){
            this.type=type; return (A) this;
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            EventFluentImpl that = (EventFluentImpl) o;
            if (apiVersion != null ? !apiVersion.equals(that.apiVersion) :that.apiVersion != null) return false;
            if (count != null ? !count.equals(that.count) :that.count != null) return false;
            if (firstTimestamp != null ? !firstTimestamp.equals(that.firstTimestamp) :that.firstTimestamp != null) return false;
            if (involvedObject != null ? !involvedObject.equals(that.involvedObject) :that.involvedObject != null) return false;
            if (kind != null ? !kind.equals(that.kind) :that.kind != null) return false;
            if (lastTimestamp != null ? !lastTimestamp.equals(that.lastTimestamp) :that.lastTimestamp != null) return false;
            if (message != null ? !message.equals(that.message) :that.message != null) return false;
            if (metadata != null ? !metadata.equals(that.metadata) :that.metadata != null) return false;
            if (reason != null ? !reason.equals(that.reason) :that.reason != null) return false;
            if (source != null ? !source.equals(that.source) :that.source != null) return false;
            if (type != null ? !type.equals(that.type) :that.type != null) return false;
            return true;
    }


    public class InvolvedObjectNestedImpl<N> extends ObjectReferenceFluentImpl<EventFluent.InvolvedObjectNested<N>> implements EventFluent.InvolvedObjectNested<N>,Nested<N>{

            private final ObjectReferenceBuilder builder;
    
            InvolvedObjectNestedImpl(){
                    this.builder = new ObjectReferenceBuilder(this);
            }
            InvolvedObjectNestedImpl(ObjectReference item){
                    this.builder = new ObjectReferenceBuilder(this, item);
            }
    
    public N and(){
            return (N) EventFluentImpl.this.withInvolvedObject(builder.build());
    }
    public N endInvolvedObject(){
            return and();
    }

}
    public class MetadataNestedImpl<N> extends ObjectMetaFluentImpl<EventFluent.MetadataNested<N>> implements EventFluent.MetadataNested<N>,Nested<N>{

            private final ObjectMetaBuilder builder;
    
            MetadataNestedImpl(ObjectMeta item){
                    this.builder = new ObjectMetaBuilder(this, item);
            }
            MetadataNestedImpl(){
                    this.builder = new ObjectMetaBuilder(this);
            }
    
    public N and(){
            return (N) EventFluentImpl.this.withMetadata(builder.build());
    }
    public N endMetadata(){
            return and();
    }

}
    public class SourceNestedImpl<N> extends EventSourceFluentImpl<EventFluent.SourceNested<N>> implements EventFluent.SourceNested<N>,Nested<N>{

            private final EventSourceBuilder builder;
    
            SourceNestedImpl(EventSource item){
                    this.builder = new EventSourceBuilder(this, item);
            }
            SourceNestedImpl(){
                    this.builder = new EventSourceBuilder(this);
            }
    
    public N and(){
            return (N) EventFluentImpl.this.withSource(builder.build());
    }
    public N endSource(){
            return and();
    }

}


}
