package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.BaseFluent;
import java.lang.String;
import com.fasterxml.jackson.annotation.JsonProperty;
import java.util.Map;
import javax.validation.Valid;
import java.util.LinkedHashMap;
import java.lang.Boolean;
import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Object;
import io.fabric8.kubernetes.api.builder.Nested;

public class FlexVolumeSourceFluentImpl<A extends FlexVolumeSourceFluent<A>> extends BaseFluent<A> implements FlexVolumeSourceFluent<A>{

    private String driver;
    private String fsType;
    private Map<String,String> options = new LinkedHashMap<String,String>();
    private Boolean readOnly;
    private VisitableBuilder<? extends LocalObjectReference,?> secretRef;

    public FlexVolumeSourceFluentImpl(){
    }
    public FlexVolumeSourceFluentImpl(FlexVolumeSource instance){
            this.withDriver(instance.getDriver()); 
            this.withFsType(instance.getFsType()); 
            this.withOptions(instance.getOptions()); 
            this.withReadOnly(instance.getReadOnly()); 
            this.withSecretRef(instance.getSecretRef()); 
    }

    public String getDriver(){
            return this.driver;
    }

    public A withDriver(String driver){
            this.driver=driver; return (A) this;
    }

    public String getFsType(){
            return this.fsType;
    }

    public A withFsType(String fsType){
            this.fsType=fsType; return (A) this;
    }

    public A addToOptions(String key,String value){
            if(key != null && value != null) {this.options.put(key, value);} return (A)this;
    }

    public A addToOptions(Map<String,String> map){
            if(map != null) { this.options.putAll(map);} return (A)this;
    }

    public A removeFromOptions(String key){
            if(key != null) {this.options.remove(key);} return (A)this;
    }

    public A removeFromOptions(Map<String,String> map){
            if(map != null) { for(Object key : map.keySet()) {this.options.remove(key);}} return (A)this;
    }

    public Map<String,String> getOptions(){
            return this.options;
    }

    public A withOptions(Map<String,String> options){
            this.options.clear();
            if (options != null) {this.options.putAll(options);} return (A) this;
    }

    public Boolean isReadOnly(){
            return this.readOnly;
    }

    public A withReadOnly(Boolean readOnly){
            this.readOnly=readOnly; return (A) this;
    }

    public LocalObjectReference getSecretRef(){
            return this.secretRef!=null?this.secretRef.build():null;
    }

    public A withSecretRef(LocalObjectReference secretRef){
            if (secretRef!=null){ this.secretRef= new LocalObjectReferenceBuilder(secretRef); _visitables.add(this.secretRef);} return (A) this;
    }

    public FlexVolumeSourceFluent.SecretRefNested<A> withNewSecretRef(){
            return new SecretRefNestedImpl();
    }

    public FlexVolumeSourceFluent.SecretRefNested<A> withNewSecretRefLike(LocalObjectReference item){
            return new SecretRefNestedImpl(item);
    }

    public FlexVolumeSourceFluent.SecretRefNested<A> editSecretRef(){
            return withNewSecretRefLike(getSecretRef());
    }

    public A withNewSecretRef(String name){
            return (A)withSecretRef(new LocalObjectReference(name));
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            FlexVolumeSourceFluentImpl that = (FlexVolumeSourceFluentImpl) o;
            if (driver != null ? !driver.equals(that.driver) :that.driver != null) return false;
            if (fsType != null ? !fsType.equals(that.fsType) :that.fsType != null) return false;
            if (options != null ? !options.equals(that.options) :that.options != null) return false;
            if (readOnly != null ? !readOnly.equals(that.readOnly) :that.readOnly != null) return false;
            if (secretRef != null ? !secretRef.equals(that.secretRef) :that.secretRef != null) return false;
            return true;
    }


    public class SecretRefNestedImpl<N> extends LocalObjectReferenceFluentImpl<FlexVolumeSourceFluent.SecretRefNested<N>> implements FlexVolumeSourceFluent.SecretRefNested<N>,Nested<N>{

            private final LocalObjectReferenceBuilder builder;
    
            SecretRefNestedImpl(){
                    this.builder = new LocalObjectReferenceBuilder(this);
            }
            SecretRefNestedImpl(LocalObjectReference item){
                    this.builder = new LocalObjectReferenceBuilder(this, item);
            }
    
    public N endSecretRef(){
            return and();
    }
    public N and(){
            return (N) FlexVolumeSourceFluentImpl.this.withSecretRef(builder.build());
    }

}


}
