package io.fabric8.kubernetes.api.model;

import io.fabric8.kubernetes.api.builder.BaseFluent;
import java.lang.String;
import javax.validation.constraints.NotNull;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import javax.validation.Valid;
import io.fabric8.kubernetes.api.model.validators.CheckObjectMeta;
import java.lang.Object;
import io.fabric8.kubernetes.api.builder.Nested;

public class ResourceQuotaFluentImpl<A extends ResourceQuotaFluent<A>> extends BaseFluent<A> implements ResourceQuotaFluent<A>{

    private String apiVersion;
    private String kind;
    private VisitableBuilder<? extends ObjectMeta,?> metadata;
    private VisitableBuilder<? extends ResourceQuotaSpec,?> spec;
    private VisitableBuilder<? extends ResourceQuotaStatus,?> status;

    public ResourceQuotaFluentImpl(){
    }
    public ResourceQuotaFluentImpl(ResourceQuota instance){
            this.withApiVersion(instance.getApiVersion()); 
            this.withKind(instance.getKind()); 
            this.withMetadata(instance.getMetadata()); 
            this.withSpec(instance.getSpec()); 
            this.withStatus(instance.getStatus()); 
    }

    public String getApiVersion(){
            return this.apiVersion;
    }

    public A withApiVersion(String apiVersion){
            this.apiVersion=apiVersion; return (A) this;
    }

    public String getKind(){
            return this.kind;
    }

    public A withKind(String kind){
            this.kind=kind; return (A) this;
    }

    public ObjectMeta getMetadata(){
            return this.metadata!=null?this.metadata.build():null;
    }

    public A withMetadata(ObjectMeta metadata){
            if (metadata!=null){ this.metadata= new ObjectMetaBuilder(metadata); _visitables.add(this.metadata);} return (A) this;
    }

    public ResourceQuotaFluent.MetadataNested<A> withNewMetadata(){
            return new MetadataNestedImpl();
    }

    public ResourceQuotaFluent.MetadataNested<A> withNewMetadataLike(ObjectMeta item){
            return new MetadataNestedImpl(item);
    }

    public ResourceQuotaFluent.MetadataNested<A> editMetadata(){
            return withNewMetadataLike(getMetadata());
    }

    public ResourceQuotaSpec getSpec(){
            return this.spec!=null?this.spec.build():null;
    }

    public A withSpec(ResourceQuotaSpec spec){
            if (spec!=null){ this.spec= new ResourceQuotaSpecBuilder(spec); _visitables.add(this.spec);} return (A) this;
    }

    public ResourceQuotaFluent.SpecNested<A> withNewSpec(){
            return new SpecNestedImpl();
    }

    public ResourceQuotaFluent.SpecNested<A> withNewSpecLike(ResourceQuotaSpec item){
            return new SpecNestedImpl(item);
    }

    public ResourceQuotaFluent.SpecNested<A> editSpec(){
            return withNewSpecLike(getSpec());
    }

    public ResourceQuotaStatus getStatus(){
            return this.status!=null?this.status.build():null;
    }

    public A withStatus(ResourceQuotaStatus status){
            if (status!=null){ this.status= new ResourceQuotaStatusBuilder(status); _visitables.add(this.status);} return (A) this;
    }

    public ResourceQuotaFluent.StatusNested<A> withNewStatus(){
            return new StatusNestedImpl();
    }

    public ResourceQuotaFluent.StatusNested<A> withNewStatusLike(ResourceQuotaStatus item){
            return new StatusNestedImpl(item);
    }

    public ResourceQuotaFluent.StatusNested<A> editStatus(){
            return withNewStatusLike(getStatus());
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            ResourceQuotaFluentImpl that = (ResourceQuotaFluentImpl) o;
            if (apiVersion != null ? !apiVersion.equals(that.apiVersion) :that.apiVersion != null) return false;
            if (kind != null ? !kind.equals(that.kind) :that.kind != null) return false;
            if (metadata != null ? !metadata.equals(that.metadata) :that.metadata != null) return false;
            if (spec != null ? !spec.equals(that.spec) :that.spec != null) return false;
            if (status != null ? !status.equals(that.status) :that.status != null) return false;
            return true;
    }


    public class MetadataNestedImpl<N> extends ObjectMetaFluentImpl<ResourceQuotaFluent.MetadataNested<N>> implements ResourceQuotaFluent.MetadataNested<N>,Nested<N>{

            private final ObjectMetaBuilder builder;
    
            MetadataNestedImpl(ObjectMeta item){
                    this.builder = new ObjectMetaBuilder(this, item);
            }
            MetadataNestedImpl(){
                    this.builder = new ObjectMetaBuilder(this);
            }
    
    public N and(){
            return (N) ResourceQuotaFluentImpl.this.withMetadata(builder.build());
    }
    public N endMetadata(){
            return and();
    }

}
    public class SpecNestedImpl<N> extends ResourceQuotaSpecFluentImpl<ResourceQuotaFluent.SpecNested<N>> implements ResourceQuotaFluent.SpecNested<N>,Nested<N>{

            private final ResourceQuotaSpecBuilder builder;
    
            SpecNestedImpl(){
                    this.builder = new ResourceQuotaSpecBuilder(this);
            }
            SpecNestedImpl(ResourceQuotaSpec item){
                    this.builder = new ResourceQuotaSpecBuilder(this, item);
            }
    
    public N endSpec(){
            return and();
    }
    public N and(){
            return (N) ResourceQuotaFluentImpl.this.withSpec(builder.build());
    }

}
    public class StatusNestedImpl<N> extends ResourceQuotaStatusFluentImpl<ResourceQuotaFluent.StatusNested<N>> implements ResourceQuotaFluent.StatusNested<N>,Nested<N>{

            private final ResourceQuotaStatusBuilder builder;
    
            StatusNestedImpl(){
                    this.builder = new ResourceQuotaStatusBuilder(this);
            }
            StatusNestedImpl(ResourceQuotaStatus item){
                    this.builder = new ResourceQuotaStatusBuilder(this, item);
            }
    
    public N and(){
            return (N) ResourceQuotaFluentImpl.this.withStatus(builder.build());
    }
    public N endStatus(){
            return and();
    }

}


}
