package io.fabric8.openshift.api.model;

import io.fabric8.kubernetes.api.builder.VisitableBuilder;
import java.lang.Boolean;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;
import javax.validation.ValidationException;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import java.util.Set;
import java.lang.Object;

public class BuildConfigSpecBuilder extends BuildConfigSpecFluentImpl<BuildConfigSpecBuilder> implements VisitableBuilder<BuildConfigSpec,BuildConfigSpecBuilder>{

    BuildConfigSpecFluent<?> fluent;
    Boolean validationEnabled;

    public BuildConfigSpecBuilder(){
            this(true);
    }
    public BuildConfigSpecBuilder(Boolean validationEnabled){
            this(new BuildConfigSpec(), validationEnabled);
    }
    public BuildConfigSpecBuilder(BuildConfigSpecFluent<?> fluent){
            this(fluent, true);
    }
    public BuildConfigSpecBuilder(BuildConfigSpecFluent<?> fluent,Boolean validationEnabled){
            this(fluent, new BuildConfigSpec(), validationEnabled);
    }
    public BuildConfigSpecBuilder(BuildConfigSpecFluent<?> fluent,BuildConfigSpec instance){
            this(fluent, instance, true);
    }
    public BuildConfigSpecBuilder(BuildConfigSpecFluent<?> fluent,BuildConfigSpec instance,Boolean validationEnabled){
            this.fluent = fluent; 
            fluent.withCompletionDeadlineSeconds(instance.getCompletionDeadlineSeconds()); 
            fluent.withOutput(instance.getOutput()); 
            fluent.withPostCommit(instance.getPostCommit()); 
            fluent.withResources(instance.getResources()); 
            fluent.withRevision(instance.getRevision()); 
            fluent.withRunPolicy(instance.getRunPolicy()); 
            fluent.withServiceAccount(instance.getServiceAccount()); 
            fluent.withSource(instance.getSource()); 
            fluent.withStrategy(instance.getStrategy()); 
            fluent.withTriggers(instance.getTriggers()); 
            this.validationEnabled = validationEnabled; 
    }
    public BuildConfigSpecBuilder(BuildConfigSpec instance){
            this(instance,true);
    }
    public BuildConfigSpecBuilder(BuildConfigSpec instance,Boolean validationEnabled){
            this.fluent = this; 
            this.withCompletionDeadlineSeconds(instance.getCompletionDeadlineSeconds()); 
            this.withOutput(instance.getOutput()); 
            this.withPostCommit(instance.getPostCommit()); 
            this.withResources(instance.getResources()); 
            this.withRevision(instance.getRevision()); 
            this.withRunPolicy(instance.getRunPolicy()); 
            this.withServiceAccount(instance.getServiceAccount()); 
            this.withSource(instance.getSource()); 
            this.withStrategy(instance.getStrategy()); 
            this.withTriggers(instance.getTriggers()); 
            this.validationEnabled = validationEnabled; 
    }

    public BuildConfigSpec build(){
            BuildConfigSpec buildable = new BuildConfigSpec(fluent.getCompletionDeadlineSeconds(),fluent.getOutput(),fluent.getPostCommit(),fluent.getResources(),fluent.getRevision(),fluent.getRunPolicy(),fluent.getServiceAccount(),fluent.getSource(),fluent.getStrategy(),fluent.getTriggers());
            validate(buildable);
            return buildable;
    }

    private <T>void validate(T item){
            if (!validationEnabled) { return; }
            Validator validator = null;
            try {
                ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
                validator = factory.getValidator();
            } catch(ValidationException e) {return;}
            Set<ConstraintViolation<T>> violations = validator.validate(item);
            if (!violations.isEmpty()) {
            throw new ConstraintViolationException(violations);
             }
    }

    public boolean equals(Object o){
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;
            if (!super.equals(o)) return false;
            BuildConfigSpecBuilder that = (BuildConfigSpecBuilder) o;
            if (fluent != null &&fluent != this ? !fluent.equals(that.fluent) :that.fluent != null &&fluent != this ) return false;

            if (validationEnabled != null ? !validationEnabled.equals(that.validationEnabled) :that.validationEnabled != null) return false;
            return true;
    }




}
